package main

import (
	"context"
	"net"
	"time"

	"github.com/sfreiberg/gotwilio"
	"gitlab.mashplace.com/mashroom/backend/common/api/health"
	"gitlab.mashplace.com/mashroom/backend/common/generic/addr"
	"gitlab.mashplace.com/mashroom/backend/common/generic/env"
	"gitlab.mashplace.com/mashroom/backend/common/generic/error/mongoerr"
	"gitlab.mashplace.com/mashroom/backend/common/generic/grpc"
	logger "gitlab.mashplace.com/mashroom/backend/common/generic/log"
	"gitlab.mashplace.com/mashroom/backend/common/generic/metrics"
	"gitlab.mashplace.com/mashroom/backend/common/generic/tracer"
	document "gitlab.mashplace.com/mashroom/backend/document/api"
	property "gitlab.mashplace.com/mashroom/backend/property/api"
	twilio "gitlab.mashplace.com/mashroom/backend/twilio/api"
	"gitlab.mashplace.com/mashroom/backend/twilio/internal/model/mongo"
	"gitlab.mashplace.com/mashroom/backend/twilio/internal/model/mongo/migrations/schema"
	"gitlab.mashplace.com/mashroom/backend/twilio/internal/service"
	"gitlab.mashplace.com/mashroom/backend/twilio/version"
)

var (
	envSelfRemote      = env.ResolveString("SELF_REMOTE", "mashroom-tenancy:10000")
	envDocumentService = document.NewDocumentServiceClient(
		grpc.Dial(env.ResolveString("DOCUMENT_SERVICE_URL", "mashroom-document:10000"), "Document"),
	)
	envPropertyService = property.NewPropertyServiceClient(
		grpc.Dial(env.ResolveString("PROPERTY_SERVICE_URL", "mashroom-property:10000"), "Property"),
	)
)

var (
	envSiteURL                   = env.ResolveString("SITE_URL", "dev.env.mashroom.com")
	envPropertyURL               = env.ResolveString("PROPERTY_URL", "https://"+envSiteURL+"/property/%s")
	envTwilioAccountSID          = env.ResolveString("TWILIO_ACCOUNT_SID", "AC14345a5251bc1e898df95ba92ef89ee0")
	envTwilioAuthToken           = env.ResolveString("TWILIO_AUTH_TOKEN", "5e51e4682d6490159a6bfa8c91468bd7")
	envTwilioMessagingServiceSID = env.ResolveString("TWILIO_MESSAGING_SERVICE_SID", "MG448a070149c53143068e7af2594b6240")
	envMediaBaseURL              = "https://" + envSiteURL + "/media/%s"
	envMobileCode                = env.ResolveString("MOBILE_CODE", "44")
	envMobilePrefix              = env.ResolveString("MOBILE_PREFIX", "+"+envMobileCode)
	envSupportPhoneNumber        = env.ResolveString("SUPPORT_PHONE_NUMBER", "+447360494104")
)

var (
	envOfficeWeekendStartHour   = env.ResolveInt("OFFICE_WEEKEND_START_HOUR", 9)
	envOfficeWeekendStartMinute = env.ResolveInt("OFFICE_WEEKEND_START_MINUTE", 0)
	envOfficeWeekendEndHour     = env.ResolveInt("OFFICE_WEEKEND_END_HOUR", 16)
	envOfficeWeekendEndMinute   = env.ResolveInt("OFFICE_WEEKEND_END_MINUTE", 0)
	envOfficeWeekdayStartHour   = env.ResolveInt("OFFICE_WEEKDAY_START_HOUR", 9)
	envOfficeWeekdayStartMinute = env.ResolveInt("OFFICE_WEEKDAY_START_MINUTE", 0)
	envOfficeWeekdayEndHour     = env.ResolveInt("OFFICE_WEEKDAY_END_HOUR", 17)
	envOfficeWeekdayEndMinute   = env.ResolveInt("OFFICE_WEEKDAY_END_MINUTE", 0)

	envWorkingHoursWeekdayStart = env.ResolveDuration(
		"WORKING_HOURS_WEEKDAY_START",
		time.Duration(envOfficeWeekdayStartHour)*time.Hour+time.Duration(envOfficeWeekdayStartMinute)*time.Minute,
	)
	envWorkingHoursWeekdayEnd = env.ResolveDuration(
		"WORKING_HOURS_WEEKDAY_END",
		time.Duration(envOfficeWeekdayEndHour)*time.Hour+time.Duration(envOfficeWeekdayEndMinute)*time.Minute,
	)
	envWorkingHoursWeekendStart = env.ResolveDuration(
		"WORKING_HOURS_WEEKEND_START",
		time.Duration(envOfficeWeekendStartHour)*time.Hour+time.Duration(envOfficeWeekendStartMinute)*time.Minute,
	)
	envWorkingHoursWeekendEnd = env.ResolveDuration(
		"WORKING_HOURS_WEEKEND_END",
		time.Duration(envOfficeWeekendEndHour)*time.Hour+time.Duration(envOfficeWeekendEndMinute)*time.Minute,
	)
)

var (
	envMongoURL      = env.ResolveString("MONGO_URL", "mongodb:27017")
	envMongoUsername = env.ResolveString("MONGO_USERNAME", "tenancy-data")
	envMongoPassword = env.ResolveString("MONGO_PASSWORD", "tenancy-data")
	envMongoAuthDB   = env.ResolveString("MONGO_AUTH", "admin")
	envMongoDB       = env.ResolveString("MONGO_DATABASE", "tenancy-data")
	envMongoSSLMode  = env.ResolveBool("MONGO_SSL_MODE", false)
)

func main() {
	ctx := context.Background()

	defer tracer.Close()

	repository, err := mongo.NewRepository(ctx, &mongo.Config{
		URL:          envMongoURL,
		Username:     envMongoUsername,
		Password:     envMongoPassword,
		Database:     envMongoDB,
		AuthSource:   envMongoAuthDB,
		MongoSSLMode: envMongoSSLMode,
	})
	if err != nil {
		panic(err)
	}

	cs := service.Connections{
		Property: envPropertyService,
		Document: envDocumentService,
	}

	srv, err := service.NewService(&service.Config{
		Connections:  cs,
		Repository:   repository,
		MediaBaseURL: envMediaBaseURL,
		WorkingHoursWeekday: service.Interval{
			Start: envWorkingHoursWeekdayStart,
			End:   envWorkingHoursWeekdayEnd,
		},
		WorkingHoursWeekend: service.Interval{
			Start: envWorkingHoursWeekendStart,
			End:   envWorkingHoursWeekendEnd,
		},
		SupportPhoneNumber: envSupportPhoneNumber,
		MobilePrefix:       envMobilePrefix,
		MobileCode:         envMobileCode,
		PropertyURL:        envPropertyURL,
		Twilio: service.Twilio{
			TwilioMessagingServiceSid: envTwilioMessagingServiceSID,
			Client:                    gotwilio.NewTwilioClient(envTwilioAccountSID, envTwilioAuthToken),
		},
	})
	if err != nil {
		panic(err)
	}

	err = repository.MigrateSchema(ctx, schema.Migrations()...)
	if err != nil {
		panic(err)
	}

	listener, err := net.Listen("tcp", addr.Grpc)
	if err != nil {
		panic(err)
	}

	grpcServer, err := grpc.NewServer("twilio", grpc.ServerOptionErrorTranslator(
		mongoerr.Translate,
	))
	if err != nil {
		panic(err)
	}

	twilio.RegisterTwilioServiceServer(grpcServer, srv)

	logger.Infoln("twilio started")

	healthServer := version.NewHealthServer()

	health.RegisterHealthServiceServer(grpcServer, healthServer)

	go func() {
		_ = metrics.ConfigureListenAndServe(grpcServer)
	}()

	go func() {
		err = srv.UploadFiles(ctx)
		if err != nil {
			panic(err)
		}
	}()

	err = grpcServer.Serve(listener)
	if err != nil {
		logger.Errorln("gRPC error", err)
	}
}
