package mongo

import (
	"context"
	"crypto/tls"
	"fmt"

	"gitlab.mashplace.com/mashroom/backend/common/generic/migrations"
	migrationsMongo "gitlab.mashplace.com/mashroom/backend/common/generic/migrations/mongo"
	"gitlab.mashplace.com/mashroom/backend/twilio/internal/model"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

const (
	serviceCollectionTwilio = "twilio"
)

// Config mongo repository config
type Config struct {
	URL          string
	Username     string
	Password     string
	Database     string
	AuthSource   string
	MongoSSLMode bool
}

type repositoryImpl struct {
	client   *mongo.Client
	database *mongo.Database
	*repositoryImplTwilio
}

func validateConfig(config *Config) error {
	if config == nil {
		return fmt.Errorf("invalid repository config")
	}

	if config.URL == "" {
		return fmt.Errorf("empty url in repository config")
	}

	if config.Username == "" {
		return fmt.Errorf("empty username in repository config")
	}

	if config.Password == "" {
		return fmt.Errorf("empty password in repository config")
	}

	if config.Database == "" {
		return fmt.Errorf("empty database in repository config")
	}

	if config.AuthSource == "" {
		return fmt.Errorf("empty database in repository config")
	}

	return nil
}

// NewRepository returns new repository instance
func NewRepository(ctx context.Context, config *Config) (model.Repository, error) {
	err := validateConfig(config)
	if err != nil {
		return nil, err
	}

	clientOptions := options.Client().ApplyURI(fmt.Sprintf("mongodb://%s", config.URL)).
		SetAuth(options.Credential{
			AuthSource:  config.AuthSource,
			Username:    config.Username,
			Password:    config.Password,
			PasswordSet: true,
		})

	if config.MongoSSLMode {
		clientOptions.SetTLSConfig(&tls.Config{})
	}

	client, err := mongo.NewClient(clientOptions)
	if err != nil {
		return nil, err
	}

	err = client.Connect(ctx)
	if err != nil {
		return nil, err
	}

	db := client.Database(config.Database)

	return &repositoryImpl{
		client:   client,
		database: db,
		repositoryImplTwilio: &repositoryImplTwilio{
			Twilio: db.Collection(serviceCollectionTwilio),
		},
	}, nil
}

func (repo *repositoryImpl) MigrateSchema(ctx context.Context, ms ...migrations.Migration) error {
	return migrations.MigrateSchema(migrationsMongo.WithInstance(ctx, repo.database), ms...)
}
