// Package service provides implementation for service APIs
package service

import (
	"bytes"
	"context"
	"fmt"
	"sync"
	"time"

	"github.com/sfreiberg/gotwilio"
	document "gitlab.mashplace.com/mashroom/backend/document/api"
	property "gitlab.mashplace.com/mashroom/backend/property/api"
	twilio "gitlab.mashplace.com/mashroom/backend/twilio/api"
	"gitlab.mashplace.com/mashroom/backend/twilio/internal/model"
	"gitlab.mashplace.com/mashroom/backend/twilio/internal/restapi"
	"gitlab.mashplace.com/mashroom/backend/twilio/voice"
)

// Connections for external services
type Connections struct {
	Property property.PropertyServiceClient
	Document document.DocumentServiceClient
}

// Interval represents [daily] interval, currently used for denoting working hours on weekends and weekdays
type Interval struct {
	Start time.Duration
	End   time.Duration
}

// Twilio client and account ID
type Twilio struct {
	TwilioMessagingServiceSid string
	Client                    *gotwilio.Twilio
}

// Config for property service
type Config struct {
	Connections         Connections
	Repository          model.Repository
	MediaBaseURL        string
	WorkingHoursWeekday Interval
	WorkingHoursWeekend Interval
	SupportPhoneNumber  string
	MobilePrefix        string
	MobileCode          string
	PropertyURL         string
	Twilio              Twilio
}

// Service contract for property service
type Service interface {
	twilio.TwilioServiceServer
	UploadFiles(ctx context.Context) error
}

// NewService returns new service instance with provided config
func NewService(config *Config) (Service, error) {
	return &serviceImpl{
		Config:    *config,
		fileNames: make(map[model.IvrStateEnum]string),
	}, nil
}

type serviceImpl struct {
	Config
	fileNames map[model.IvrStateEnum]string
	m         sync.RWMutex
}

var ivrStateNames = map[model.IvrStateEnum]string{
	model.IvrStateWelcome:                "welcome-message",
	model.IvrStateEnterPropertyRef:       "enter-propref",
	model.IvrStateEnterPropertyRefRepeat: "ID-incorrect",
	model.IvrStatePropertyRefIncorrect:   "ID-incorrect-twice",
	model.IvrStateMorePropertyInfo:       "info-for-property",
	model.IvrStateMobile:                 "book-mobile",
	model.IvrStateMobileCorrect:          "book-mobile-corect",
	model.IvrStateBookAgain:              "book-again",
	model.IvrStateAnotherQuery:           "other-queries-working-hours",
	model.IvrStateHowToList:              "How-to-list-outside-hours",
	model.IvrStateOutro:                  "thanks-for-calling",
	model.IvrStateOutroOutOfHours:        "outro-out-of-hours",
}

func (srv *serviceImpl) UploadFiles(ctx context.Context) error {
	srv.m.Lock()
	defer srv.m.Unlock()

	for k, v := range ivrStateNames {
		fname := v + ".mp3"

		bs, err := voice.FS.ReadFile(fname)
		if err != nil {
			return err
		}

		f, err := document.UploadFile(
			ctx,
			srv.Connections.Document,
			fname,
			"audio/mpeg",
			&document.FileAttribute{
				Data: map[string]string{
					document.FileAttributeDocumentType.String(): document.TypeOther,
				},
			},
			bytes.NewReader(bs),
		)
		if err != nil {
			return err
		}

		srv.fileNames[k] = f.GetId()
	}

	return nil
}

func (srv *serviceImpl) voiceMedia(state model.IvrStateEnum) *restapi.Play {
	srv.m.RLock()
	defer srv.m.RUnlock()

	if media, ok := srv.fileNames[state]; ok {
		return &restapi.Play{
			Loop: 1,
			Text: fmt.Sprintf(srv.MediaBaseURL, media),
		}
	}

	return nil
}
