package service

import (
	"context"
	"encoding/xml"
	"fmt"
	"net/url"
	"path"
	"strings"
	"time"

	"gitlab.mashplace.com/mashroom/backend/common/api/grpcerr"
	"gitlab.mashplace.com/mashroom/backend/common/generic/log"
	property "gitlab.mashplace.com/mashroom/backend/property/api"
	twilio "gitlab.mashplace.com/mashroom/backend/twilio/api"
	"gitlab.mashplace.com/mashroom/backend/twilio/internal/model"
	"gitlab.mashplace.com/mashroom/backend/twilio/internal/restapi"
	"gitlab.mashplace.com/mashroom/backend/twilio/voice"
)

func (srv *serviceImpl) lookupState(ctx context.Context, sid string) (*model.IVRState, error) {
	m, err := srv.Repository.IVRStateGetBySID(ctx, sid)
	if err != nil {
		return nil, err
	}

	if m == nil {
		m = &model.IVRState{
			CallSid: sid,
			State:   model.IvrStateWelcome,
		}
	}

	return m, nil
}

func (srv *serviceImpl) workingHours() bool {
	var hours *Interval

	now := time.Now()

	switch now.Weekday() {
	case time.Sunday:
		return false
	case time.Saturday:
		hours = &srv.WorkingHoursWeekend
	default:
		hours = &srv.WorkingHoursWeekday
	}

	cur := time.Since(now.Truncate(time.Hour * 24))

	return cur > hours.Start && cur < hours.End
}

func (srv *serviceImpl) response(state model.IvrStateEnum) *restapi.Response {
	switch state {
	case model.IvrStateMorePropertyInfo:
		if srv.workingHours() {
			return &restapi.Response{
				Play: []*restapi.Play{
					srv.voiceMedia(state),
				},
				Gather: &restapi.Gather{
					Play: srv.voiceMedia(model.IvrStateAnotherQuery),
				},
			}
		}

		return &restapi.Response{
			Play: []*restapi.Play{
				srv.voiceMedia(state),
				srv.voiceMedia(model.IvrStateOutroOutOfHours),
			},
		}
	case model.IvrStateEnterPropertyRef, model.IvrStateEnterPropertyRefRepeat:
		return &restapi.Response{
			Gather: &restapi.Gather{
				Play:      srv.voiceMedia(state),
				Timeout:   "15",
				NumDigits: "6",
			},
		}
	case model.IvrStateMobile:
		return &restapi.Response{
			Gather: &restapi.Gather{
				Play:      srv.voiceMedia(state),
				Timeout:   "15",
				NumDigits: "11",
			},
		}
	case model.IvrStateHowToList:
		return &restapi.Response{
			Play: []*restapi.Play{
				srv.voiceMedia(state),
				srv.voiceMedia(model.IvrStateOutroOutOfHours),
			},
		}
	case model.IvrStateOutro, model.IvrStateOutroOutOfHours, model.IvrStatePropertyRefIncorrect:
		return &restapi.Response{
			Play: []*restapi.Play{
				srv.voiceMedia(state),
			},
		}
	default:
		return &restapi.Response{
			Gather: &restapi.Gather{
				Play:      srv.voiceMedia(state),
				Timeout:   "15",
				NumDigits: "1",
			},
		}
	}
}

func (srv *serviceImpl) parseMobileNumber(mobileNumber string) (string, error) {
	if len(mobileNumber) == 0 {
		return "", grpcerr.New(grpcerr.InvalidData, "empty mobile number")
	}

	if string(mobileNumber[0]) == "+" {
		return mobileNumber, nil
	}

	if len(mobileNumber) == 11 && string(mobileNumber[0]) == "7" {
		mobileNumber = "+" + mobileNumber
		return mobileNumber, nil
	}

	if string(mobileNumber[0]) == "0" {
		if len(mobileNumber) >= 10 {
			return srv.MobilePrefix + mobileNumber[1:], nil
		}

		return srv.MobilePrefix + mobileNumber, nil
	}

	if strings.HasPrefix(mobileNumber, srv.MobileCode) {
		if len(mobileNumber) >= 11 {
			return "+" + strings.Replace(mobileNumber, srv.MobileCode, "", 1), nil
		}

		return srv.MobilePrefix + mobileNumber, nil
	}

	return mobileNumber, nil
}

func (srv *serviceImpl) sendSMS(mobileNumber string, prop []string) error {
	body := "Book your viewing by clicking on this link: "

	var urls []string

	for _, p := range prop {
		urls = append(urls, fmt.Sprintf(srv.PropertyURL, p))
	}

	body += strings.Join(urls, ",")

	_, ex, err := srv.Twilio.Client.SendSMSWithCopilot(srv.Twilio.TwilioMessagingServiceSid, mobileNumber, body, "", "")
	if ex != nil {
		return grpcerr.New(grpcerr.ProviderError, ex.Error())
	}

	if err != nil {
		return grpcerr.New(grpcerr.ProviderError, err.Error())
	}

	return nil
}

func (srv *serviceImpl) lookupProperty(ctx context.Context, ivr string) (string, error) {
	if len(ivr) != 6 {
		return "", grpcerr.New(grpcerr.InvalidData, "ivr value not equals 6")
	}

	resp, err := srv.Connections.Property.PropertyIDByIVR(ctx, &property.PropertyIDByIVRRequest{
		Ivr: ivr,
	})

	return resp.GetPropertyId(), err
}

func (srv *serviceImpl) IVRCallbackHandler(
	ctx context.Context,
	req *twilio.IVRCallbackHandlerRequest,
) (resp *twilio.IVRCallbackHandlerResponse, err error) {
	values, err := url.ParseQuery(req.GetData())
	if err != nil {
		return nil, err
	}

	callSID := values.Get("CallSid")

	log.WithField("CallSid", callSID).Debug("parsed caller SID")

	if callSID == "" {
		return nil, grpcerr.New(grpcerr.InvalidData, "forbidden for callSid empty")
	}

	m, err := srv.lookupState(ctx, callSID)
	if err != nil {
		return nil, err
	}

	old := *m

	response := srv.processStateChange(ctx, m, values.Get("Digits"), values.Get("Caller"))

	switch {
	case m.Delete:
		err = srv.Repository.IVRStateRemoveBySID(ctx, m.CallSid)
	case m.Changed(&old):
		err = srv.Repository.IVRStateSave(ctx, m)
	}

	if err != nil {
		return nil, err
	}

	if response == nil {
		return nil, grpcerr.New(grpcerr.InvalidData, "unrecognised option")
	}

	bs, err := xml.Marshal(response)
	if err != nil {
		return nil, err
	}

	return &twilio.IVRCallbackHandlerResponse{
		Data: string(bs),
	}, nil
}

func (srv *serviceImpl) connectToSupport() *restapi.Response {
	return &restapi.Response{
		Dial: &restapi.Dial{
			Text: srv.SupportPhoneNumber,
		},
	}
}

func (srv *serviceImpl) processStateChangeWelcome(m *model.IVRState) *restapi.Response {
	m.State = model.IvrStateWelcomeResponse

	return srv.response(model.IvrStateWelcome)
}

func (srv *serviceImpl) processStateChangeWelcomeResponse(m *model.IVRState, choice string) *restapi.Response {
	switch choice {
	case "1":
		m.State = model.IvrStateEnterPropertyRefResponse

		return srv.response(model.IvrStateEnterPropertyRef)
	case "2":
		// More info on property
		m.State = model.IvrStateAnotherQuery

		return srv.response(model.IvrStateMorePropertyInfo)
	case "3":
		if srv.workingHours() {
			return srv.connectToSupport()
		}

		return srv.response(model.IvrStateHowToList)
	case "4":
		// Other
		if srv.workingHours() {
			return srv.connectToSupport()
		}

		return srv.response(model.IvrStateOutroOutOfHours)
	case "5":
		// Repeat
		m.State = model.IvrStateWelcome

		return srv.response(model.IvrStateWelcome)
	}

	return nil
}

func (srv *serviceImpl) processStateChangePropertyRef(
	ctx context.Context,
	m *model.IVRState,
	digits string,
) *restapi.Response {
	prop, err := srv.lookupProperty(ctx, digits)
	if err != nil {
		if m.State == model.IvrStateEnterPropertyRefRepeat {
			if srv.workingHours() {
				return srv.connectToSupport()
			}

			return srv.response(model.IvrStatePropertyRefIncorrect)
		}

		m.State = model.IvrStateEnterPropertyRefRepeat

		return srv.response(model.IvrStateEnterPropertyRefRepeat)
	}

	m.PropertyIDs = append(m.PropertyIDs, prop)
	m.State = model.IvrStateMobileCorrect

	return srv.response(model.IvrStateMobileCorrect)
}

func (srv *serviceImpl) processStateChangeMobileCorrect(
	m *model.IVRState,
	digits, caller string,
) *restapi.Response {
	switch digits {
	case "1": // correct
		mobileNumber, err := srv.parseMobileNumber(caller)
		if err != nil {
			m.State = model.IvrStateMobile

			return srv.response(model.IvrStateMobile)
		}

		err = srv.sendSMS(mobileNumber, m.PropertyIDs)
		if err != nil {
			m.State = model.IvrStateMobile

			return srv.response(model.IvrStateMobile)
		}

		m.State = model.IvrStateBookAgain

		return srv.response(model.IvrStateBookAgain)
	case "2": // not correct
		m.State = model.IvrStateMobile

		return srv.response(model.IvrStateMobile)
	default:
		return srv.response(model.IvrStateMobileCorrect)
	}
}

func (srv *serviceImpl) processStateChangeMobile(
	m *model.IVRState,
	digits string,
) *restapi.Response {
	mobileNumber, err := srv.parseMobileNumber(digits)
	if err != nil {
		return srv.response(model.IvrStateMobile)
	}

	err = srv.sendSMS(mobileNumber, m.PropertyIDs)
	if err != nil {
		srv.response(model.IvrStateMobile)
	}

	m.State = model.IvrStateBookAgain

	return srv.response(model.IvrStateBookAgain)
}

func (srv *serviceImpl) processStateChangeBookAgain(
	m *model.IVRState,
	digits string,
) *restapi.Response {
	if digits == "1" { // yes
		m.State = model.IvrStateEnterPropertyRefResponse

		return srv.response(model.IvrStateEnterPropertyRef)
	}

	if srv.workingHours() {
		m.State = model.IvrStateAnotherQuery

		return srv.response(model.IvrStateAnotherQuery)
	}

	m.Delete = true

	return srv.response(model.IvrStateOutro)
}

func (srv *serviceImpl) processStateChangeAnotherQuery(
	m *model.IVRState,
	digits string,
) *restapi.Response {
	if digits == "1" { // yes
		return srv.connectToSupport()
	}

	m.Delete = true

	return srv.response(model.IvrStateOutro)
}

func (srv *serviceImpl) processStateChange(
	ctx context.Context,
	m *model.IVRState,
	digits, caller string,
) *restapi.Response {
	switch m.State {
	case model.IvrStateWelcome:
		return srv.processStateChangeWelcome(m)
	case model.IvrStateWelcomeResponse:
		return srv.processStateChangeWelcomeResponse(m, digits)
	case model.IvrStateEnterPropertyRefResponse, model.IvrStateEnterPropertyRefRepeat:
		return srv.processStateChangePropertyRef(ctx, m, digits)
	case model.IvrStateMobileCorrect:
		return srv.processStateChangeMobileCorrect(m, digits, caller)
	case model.IvrStateMobile:
		return srv.processStateChangeMobile(m, digits)
	case model.IvrStateBookAgain:
		return srv.processStateChangeBookAgain(m, digits)
	case model.IvrStateAnotherQuery:
		return srv.processStateChangeAnotherQuery(m, digits)
	}

	return nil
}

func (srv *serviceImpl) IVRFileHandler(
	ctx context.Context,
	req *twilio.IVRFileHandlerRequest,
) (*twilio.IVRFileHandlerResponse, error) {
	fname := path.Base(req.GetFile())

	bs, err := voice.FS.ReadFile(fname)
	if err != nil {
		return nil, err
	}

	return &twilio.IVRFileHandlerResponse{
		File: fname,
		Data: bs,
		Mime: "audio/mpeg",
	}, nil
}
